/****************************************************************************
**
** Copyright (C) 2007-2007 Trolltech ASA. All rights reserved.
**
** This file is part of the demonstration applications of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** In addition, as a special exception, Trolltech gives you certain
** additional rights. These rights are described in the Trolltech GPL
** Exception version 1.0, which can be found at
** http://www.trolltech.com/products/qt/gplexception/ and in the file
** GPL_EXCEPTION.txt in this package.
**
** In addition, as a special exception, Trolltech, as the sole copyright
** holder for Qt Designer, grants users of the Qt/Eclipse Integration
** plug-in the right for the Qt/Eclipse Integration to link to
** functionality provided by Qt Designer and its related libraries.
**
** Trolltech reserves all rights not expressly granted herein.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "browsermainwindow.h"

#include <QtCore/QLibraryInfo>
#include <QtCore/QLocale>
#include <QtCore/QPointer>
#include <QtCore/QTranslator>
#include <QtCore/QUrl>
#include <QtCore/QDebug>

#include <QtGui/QApplication>
#include <QtGui/QDesktopServices>

#include <QtWebKit/QWebSettings>

class BrowserApplication : public QApplication
{
    Q_OBJECT
public:
    BrowserApplication(int &argc, char** argv)
        : QApplication(argc, argv)
    {
#if defined(Q_WS_MAC)
        QApplication::setQuitOnLastWindowClosed(false);
#else
        QApplication::setQuitOnLastWindowClosed(true);
#endif

        QString localSysName = QLocale::system().name();
        installTranslator(QLatin1String("browser_") + localSysName);
        installTranslator(QLatin1String("qt_") + localSysName);
    }

#if defined(Q_WS_MAC)
    bool event(QEvent* event)
    {
        if (event->type() == QEvent::ApplicationActivate && m_mainWindow && !m_mainWindow->isMinimized())
            m_mainWindow->show();

        return QApplication::event(event);
    }
#endif

    void setMainWindow(BrowserMainWindow* mainWindow)
    {
        m_mainWindow = mainWindow;
    }

public Q_SLOTS:
    void openUrl(const QUrl &url)
    {
        qDebug() << "openUrl" << url;
        m_mainWindow->loadPage(url.toString());
    }

private:
    void installTranslator(const QString& name)
    {
        QTranslator* translator = new QTranslator(qApp);
        translator->load(name, QLibraryInfo::location(QLibraryInfo::TranslationsPath));
        qApp->installTranslator(translator);
    }

    QPointer<BrowserMainWindow> m_mainWindow;
};


int main(int argc, char** argv)
{
    BrowserApplication app(argc, argv);
    QCoreApplication::setOrganizationName("Trolltech");
    QCoreApplication::setApplicationName("demobrowser");

    QWebSettings::setIconDatabaseEnabled(true, app.applicationDirPath());

    BrowserMainWindow *browser = new BrowserMainWindow;
    app.setMainWindow(browser);

    QDesktopServices::setUrlHandler("http", &app, "openUrl");

    browser->show();

    return app.exec();
}

#include "main.moc"
