/****************************************************************************
**
** Copyright (C) 2006-2007 Trolltech ASA. All rights reserved.
**
** This file is part of the QtXMLPatterns module of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** In addition, as a special exception, Trolltech gives you certain
** additional rights. These rights are described in the Trolltech GPL
** Exception version 1.0, which can be found at
** http://www.trolltech.com/products/qt/gplexception/ and in the file
** GPL_EXCEPTION.txt in this package.
**
** In addition, as a special exception, Trolltech, as the sole copyright
** holder for Qt Designer, grants users of the Qt/Eclipse Integration
** plug-in the right for the Qt/Eclipse Integration to link to
** functionality provided by Qt Designer and its related libraries.
**
** Trolltech reserves all rights not expressly granted herein.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef Patternist_FirstItemPredicate_H
#define Patternist_FirstItemPredicate_H

#include "qsinglecontainer_p.h"

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

namespace QPatternist
{
    /**
     * @short A predicate that always selects the first item from its sequence.
     *
     * FirstItemPredicate corresponds exactly to the predicate
     * in the expression <tt>input[1]</tt>.
     *
     * @author Frans Englich <fenglich@trolltech.com>
     * @ingroup Patternist_expressions
     */
    class FirstItemPredicate : public SingleContainer
    {
    public:
        /**
         * Creates a FirstItemPredicate that filters @p source.
         */
        FirstItemPredicate(const Expression::Ptr &source);

        /**
         * @returns the first item, if any, from evaluating the source expression.
         */
        virtual Item evaluateSingleton(const DynamicContext::Ptr &) const;

        /**
         * @returns a list containing one CommonSequenceTypes::ZeroOrMoreItems instance.
         */
        virtual SequenceType::List expectedOperandTypes() const;

        /**
         * @returns a SequenceType where the item type is the same as the source expression
         * and where the cardinality is either Cardinality::zeroOrOne() or Cardinality::exactlyOne(),
         * depending on the source expression.
         */
        virtual SequenceType::Ptr staticType() const;
        virtual ExpressionVisitorResult::Ptr accept(const ExpressionVisitor::Ptr &visitor) const;

        /**
         * Rewrites <tt>expression[1][1]</tt> into <tt>expression[1]</tt>.
         */
        virtual Expression::Ptr compress(const StaticContext::Ptr &context);

        /**
         * @returns always IDFirstItemPredicate.
         */
        virtual ID id() const;
    };
}

QT_END_NAMESPACE

QT_END_HEADER

#endif
