/****************************************************************************
**
** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
**
** This file is part of the Qt Linguist of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://trolltech.com/products/qt/licenses/licensing/opensource/
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
** or contact the sales department at sales@trolltech.com.
**
** In addition, as a special exception, Trolltech gives you certain
** additional rights. These rights are described in the Trolltech GPL
** Exception version 1.0, which can be found at
** http://www.trolltech.com/products/qt/gplexception/ and in the file
** GPL_EXCEPTION.txt in this package.
**
** In addition, as a special exception, Trolltech, as the sole copyright
** holder for Qt Designer, grants users of the Qt/Eclipse Integration
** plug-in the right for the Qt/Eclipse Integration to link to
** functionality provided by Qt Designer and its related libraries.
**
** Trolltech reserves all rights not expressly granted herein.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "recentfiles.h"
#include <QSettings>
#include <QString>
#include <QStringList>
#include <QFileInfo>

QT_BEGIN_NAMESPACE

const char * const CONFIGKEY = "RecentlyOpenedFiles";

RecentFiles::RecentFiles(const QString &settingsPrefix, const int maxEntries)
        :  m_maxEntries(maxEntries), m_settingsPrefix(settingsPrefix)
{
}

void RecentFiles::addFile(const QString &name)
{
    const QString filePath = QFileInfo(name).absoluteFilePath();

    if (!m_strList.isEmpty() && m_strList.first() == filePath)
        return;

    m_strList.removeOne(filePath);
    if (m_strList.count() >= m_maxEntries)
        m_strList.removeLast();
    m_strList.prepend(filePath);
    emit changed();
}

void RecentFiles::readConfig()
{
    QSettings config;
    m_strList = config.value(m_settingsPrefix + QLatin1String(CONFIGKEY)).toStringList();
    emit changed();
}

void RecentFiles::writeConfig() const
{
    QSettings config;
    config.setValue(m_settingsPrefix + QLatin1String(CONFIGKEY), m_strList);
}

QT_END_NAMESPACE
